<?php
namespace app\common\model;
use think\Model;
use think\Db;

class Ewm extends Model
{
	// 直接使用配置参数名
	private $_table;
	private $tableOk = false;
	function __construct()
	{
		parent::__construct();
		$this->_table = config('database.db_config_mysql_1')['prefix'].'ewm_'.date('Y');
		if($this->tableOk == true)
			return;
		$createSql = <<<EOT
create table if not exists {$this->_table}(
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `no` varchar(20) NOT NULL COMMENT '二维码编码',
  `zhuyuanhao` varchar(12) NOT NULL COMMENT '住院号',
  `shenfenzheng` varchar(18) NOT NULL COMMENT '身份证号',
  `name` varchar(16) NOT NULL COMMENT '姓名',
  `mobile` char(11) NOT NULL COMMENT '手机号码',
  `p_mobile` varchar(11) NOT NULL COMMENT '患者手机后4位',
  `addtime` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP COMMENT '创建时间',
  `is_self` tinyint(1) NOT NULL DEFAULT '0' COMMENT '是不是患者本人:0不是,1是',
  PRIMARY KEY (`id`),
  UNIQUE KEY `no` (`no`),
  KEY `i_no` (`no`),
  KEY `i_zhuyuanhao` (`zhuyuanhao`),
  KEY `i_addtime` (`addtime`),
  KEY `i_shenfenzheng` (`shenfenzheng`),
  KEY `i_mobile` (`mobile`),
  KEY `i_name` (`name`),
  KEY `i_p_mobile` (`p_mobile`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 COMMENT='二维码表按年分表';
EOT;
		$result = Db::connect(config('database.db_config_mysql_1'))->query($createSql);
		$this->tableOk = true;
	}
	//录入二维码信息
	public function insertEwm($params,$patientinfo)
	{
		$return = [];  
		$noNormalKS = $this->noNormalKS();
		$in_nnk = in_array($patientinfo['BINGQUCODE'],$noNormalKS);
		if($in_nnk)
		{
			$authority = model('Authority');
			$ewm['shenfenzheng'] = $params['shenfenzheng'];
			$rules = $authority->getAuthRule($patientinfo,$ewm);
			if($rules['b'] == '' && $rules['s'] == '' && $rules['d'] == '')
			{
				$error = '此患者所属病区还未开放探视权限';
				return ['res'=>true,'data'=>'','error'=>$error];
			}	
		}
		//初始化权限
		$this->auth_init($params['zhuyuanhao'],$patientinfo,$in_nnk);

		$db = Db::connect(config('database.db_config_mysql_1'));
		$db->startTrans();
		//*
		try{
		//*/
			/**2.生成二维码并记录**/
			$today = date('Y-m-d');
			$nos = $db->table($this->_table)->where("zhuyuanhao='{$params['zhuyuanhao']}' and shenfenzheng='{$params['shenfenzheng']}' and addtime>='{$today}'")->limit($params['persons'])->field('no,name')->select();
			$is_self = 	$params['shenfenzheng'] == $patientinfo['SHENFENZHENG'] ? 1 :  0;

			//访客名字处理
			
	   		$countToday =  $db->table($this->_table)->where("addtime>='{$today}'")->count();
	   		$name = $params['name']  ? $params['name'] : '访客'.$countToday;

			$persons = [];
			for($k=0;$k<$params['persons'];$k++)
			{
				$persons[$k] = '';
			}
			$nl = count($nos);
			$max_len = $nl >= $params['persons'] ? $nl : $params['persons'];
			for($i=0;$i<$max_len;$i++) {
				if(isset($nos[$i]) && isset($persons[$i]))
					 continue;
				if(!isset($persons[$i]) && isset($nos[$i]))
				{
					unset($nos[$i]);
					continue;
				}
				$no = createEwmNo();
				$is_self = 	$params['shenfenzheng'] == $patientinfo['SHENFENZHENG'] ? 1 :  0;
				$name = ($i > 0 ? ('访客'.($countToday+$i)): $name);
				$db->table($this->_table)->insert([
					'no' => $no,
					'zhuyuanhao' => $params['zhuyuanhao'],
					'shenfenzheng' => $params['shenfenzheng'],
					'p_mobile' => $params['p_mobile'] ?? '未提供',
					'name' => $name,
					'mobile' => $params['mobile'] ?? '',
					'is_self' => $is_self,
				]);
				$nos[$i] = [
					'no'=>$no,
					'name'=>$name
				];
			}
			//患者-家属表
			$id = $db->name('dependents')->where("zhuyuanhao='{$params['zhuyuanhao']}' and shenfenzheng='{$params['shenfenzheng']}'")->value('id');
			if(empty($id))
			{
				$db->name('dependents')->insert([
					'zhuyuanhao' => $params['zhuyuanhao'],
					'shenfenzheng' => $params['shenfenzheng'],
					'name' => $name,
					'mobile' => $params['mobile'] ?? '',
				]);
			}
			$db->commit();
			return ['res'=>true,'data'=>['row'=>$nos],'error'=>''];
		//*
		}
		catch (\Exception $e) {
    		$db->rollback();
			return ['res'=>false,'data'=>'','error'=>''];
		}//*/
	}
	//根据no查询zhuyuanhao
	public function getZyhByNo($no)
	{
		return Db::connect(config('database.db_config_mysql_1'))->table($this->_table)->where('no',$no)->field('zhuyuanhao,shenfenzheng,addtime')->find();

	}
	//初始化权限
	public function auth_init($zhuyuanhao,$patientinfo,$in_nnk = 'SAOMAIN')
	{
		if($in_nnk == 'SAOMAIN')
		{
			$noNormalKS = $this->noNormalKS();
			$in_nnk = in_array($patientinfo['BINGQUCODE'],$noNormalKS);
		}
		$db = Db::connect(config('database.db_config_mysql_1'));
		$db->startTrans();
		//*
		try{
		// */
			/**1.初始化患者信息和权限**/

			//病区权限
			//病区没有权限设置普通探视权限(1)
			$id = $db->name('object_authority')->where("object_id='{$patientinfo['BINGQUCODE']}' and type=2")->value('id');
			if(empty($id) && !$in_nnk)
			{
				$db->name('object_authority')->insert([
					'authority_id' => 1,
					'object_id' => $patientinfo['BINGQUCODE'],
					'type' => 2,
				]);
			}
			//患者权限
			//权限优先级 入院当天(超级权限)>出院(近2天)>手术>重症>检查(近3天)>陪住
			$pa_row = $db->name('patient_authority')->where("zhuyuanhao='{$zhuyuanhao}'")->field('id,is_out,is_die,is_operation_z,is_icu,is_pz,is_jc,in_status,pz_status,jc_status,rule')->find();

			$pa_row['pz_status'] = $pa_row['pz_status'] ?? 1;//是否启用,表里默认1
			$pa_row['jc_status'] = $pa_row['jc_status'] ?? 1;//是否启用,表里默认1
			//组织更新/新增数据
			$pa_data = [
				'kid' => $patientinfo['KESHICODE'],
				'bid' => $patientinfo['BINGQUCODE'],
				'is_out' => $patientinfo['YNOUT'],
				'out_date' => $patientinfo['OUTDATE'] ?? null,
				'is_icu' => $patientinfo['YN_ICU'],
				'is_jc' => $patientinfo['YN_JC'],
				'is_pz' => $patientinfo['VISIT_SPECIAL']['YNSTAY'] ?? 0,
				'is_operation_z' => $patientinfo['YN_SHOUSHU'],
				'operation_z_date' => $patientinfo['SHOUSHURQ'] ?? null,
				'is_operation' => $patientinfo['YN_SHOUSHU_NOW'],
				'visit_cause' => $patientinfo['VISIT_SPECIAL']['VISITCAUSE'] ?? null,//探视原因
				'b_auth' => 0,//继承病区的权限
			];
			if(substr($patientinfo['IN_TIME'],0,10) == date('Y-m-d'))//患者入住当日不限使用次数,设置超级权限
			{
				//id为2超级权限
				$authority_id = 2;
			}
	        else if(!empty($patientinfo['OUTDATE']) && $patientinfo['OUTDATE'] > date('Y-m-d',strtotime('-2 day')))//出院判断2天有效
	        {
	            $authority_id = 8;
	        }
			else if($patientinfo['YN_SHOUSHU_NOW'] == 1)//手术
			{
				$authority_id = 3;
			}
			else if($patientinfo['YN_ICU'] == 1)//ICU
			{
				$authority_id = 6;
			}
			else if($patientinfo['YN_JC'] == 1 && $pa_row['jc_status'] == 1)//检查
			{
				$authority_id = 4;
			}
			else if($patientinfo['VISIT_SPECIAL']['YNSTAY'] == 1 && $pa_row['pz_status'] == 1)//陪住
			{
				$authority_id = 5;
			}
			else//其他走普通权限 
			{
				$authority_id = 1;
			}
			//说明有非普通权限
			if($authority_id != 1)
			{
				$pa_data['authority_id'] = $authority_id;
			}
			else
			{
				$pa_data['authority_id'] = 1;//恢复成普通权限
			}

			//开始数据库操作
			if(empty($pa_row['id']))//插入
			{
				$pa_data['zhuyuanhao'] = $zhuyuanhao;

				$db->name('patient_authority')->insert($pa_data);
			}
			else//更新
			{
				$db->name('patient_authority')->where("id={$pa_row['id']}")->update($pa_data);
			}
			$db->commit();
		//*
		}
		catch (\Exception $e) {
    		$db->rollback();
		}// */
	}
	//不设常规探视科室
	private function noNormalKS()
	{
		//重症监护室、血液透析病房、血液肿瘤、感染病房、新生儿室、产房、产科
		return [
			/*
			11,//神外ICU(旧)
			22,//综合ICU(旧)
			1011,//神外ICU(新)
			1022,//重症医学科1病区
			1394,//国际部重症医学科病区
			1395,//重症医学科2病区
			1396,//重症医学科3病区
			298,//神经重症医学科(旧)
			1298,//神经重症医学科病区
			1392,//心脏重症监护病区
			1393,//呼吸重症监护病区

			104,//血液透析特病专用

			224,//肿瘤外科(旧)
			227,//肿瘤科
			365,//神经肿瘤科病区(旧)
			1224,//肿瘤外科(新)
			1227,//肿瘤科(新)
			1418,//研-神经肿瘤综合病区
			1014,//神经外科肿瘤6病区
			1013,//神经外科肿瘤3病区
			1009,//神经外科肿瘤1病区
			1008,//神经外科肿瘤2病区
			1006,//神经外科肿瘤4病区
			1365,//神经肿瘤综合治疗病区
			1385,//神经外科肿瘤5病区
			1406,//神经外科肿瘤7病区

			1222,//感染科病区
			222,//感染科(旧)
			337,//神经感染与免疫科(旧)
			1337,//神经感染与免疫科病区
			1417,//研-神经感染免疫病区

			1407,//新生儿室

			112,//产科病区(旧)
			1112//产科病区
			*/
		];
	}
}
